﻿/*
Copyright © Jonathan Schnittger 2013 
Do not alter, reproduce or redistribute without attribution
*/
using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Coding.Exercise.Patterns.Database
{
    /// <summary>
    /// Database connection factory singleton
    /// </summary>
    public class DatabaseFactory
    {
        /// <summary>
        /// Default command timeout
        /// </summary>
        private const int COMMAND_TIMEOUT = 30000;

        /// <summary>
        /// Singleton factory instance
        /// </summary>
        private static readonly DatabaseFactory _factory = new DatabaseFactory();

        /// <summary>
        /// Returns the currently available database providers
        /// </summary>
        public Dictionary<string, DatabaseProvider> Providers { get; private set; }

        /// <summary>
        /// Returns the singleton instance of the factory
        /// </summary>
        /// <returns></returns>
        public static DatabaseFactory GetInstance()
        {
            return _factory;
        }

        /// <summary>
        /// Initializes the class
        /// </summary>
        private DatabaseFactory()
        {
            this.Providers = new Dictionary<string, DatabaseProvider>();

            //Get the current location
            string location = System.Reflection.Assembly.GetExecutingAssembly().Location;
            string path = Path.GetDirectoryName(location);

            var idbconnection = typeof(IDbConnection);

            //Get the .Net framework location
            List<string> dlls = new List<string>();
            string dotNetPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Windows), @"Microsoft.NET\Framework" + (Environment.Is64BitProcess ? "64" : "") + @"\v" + Environment.Version.ToString(3));

            //Load the data dlls
            dlls.AddRange(Directory.GetFiles(dotNetPath, "*data.dll"));//load .net sql server
            dlls.AddRange(Directory.GetFiles(path, "*data*.dll"));

            //For each dll, check and see if it implements the IDbConnection interface
            //If it does, create an instance of it and add it to the provider collection
            foreach (string dll in dlls)
            {
                var assembly = Assembly.LoadFile(dll);

                if (DatabaseProvider.IsDatabaseProvider(assembly))
                {
                    DatabaseProvider provider = new DatabaseProvider(assembly);
                    this.Providers.Add(provider.Name, provider);
                }
            }
        }


        /// <summary>
        /// Creates a connection object
        /// </summary>
        /// <param name="providerName">Name of the connection provider</param>
        /// <param name="connectionString">The connection string</param>
        /// <param name="exError">Out exception object</param>
        /// <returns>Connection object</returns>
        public IDbConnection CreateConnection(string providerName, string connectionString, out Exception exError)
        {
            IDbConnection returnValue = null;
            exError = null;

            try
            {
                if (!this.Providers.Keys.Contains(providerName))
                    throw new Exception("Unknown provider " + providerName);

                DatabaseProvider provider = this.Providers[providerName];
                returnValue = provider.CreateConnection();
                returnValue.ConnectionString = connectionString;
            }
            catch (Exception ex)
            {
                exError = ex;
            }

            return returnValue;
        }
    }
}