﻿/*
Copyright © Jonathan Schnittger 2013 
Do not alter, reproduce or redistribute without attribution
*/
using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Coding.Exercise.Patterns.Database
{
    /// <summary>
    /// Database provider class
    /// It defines the main interface types needed and configuration
    /// </summary>
    public class DatabaseProvider
    {
        /// <summary>
        /// IDbConnection interface
        /// </summary>
        private static readonly Type CONNECTION_TYPE = typeof(IDbConnection);

        /// <summary>
        /// The name of the provider
        /// </summary>
        public string Name { get; private set; }

        /// <summary>
        /// The full name including namespace, version and guid
        /// </summary>
        public string FullName { get; private set; }

        /// <summary>
        /// The in-memory dll
        /// </summary>
        public Assembly Assembly { get; private set; }

        /// <summary>
        /// Type used for connection initialization
        /// </summary>
        public Type ConnectionType { get; private set; }

        /// <summary>
        /// Constructor for the database provider
        /// </summary>
        /// <param name="assembly">The in-memory dll</param>
        public DatabaseProvider(Assembly assembly)
        {
            this.Assembly = assembly;

            this.Name = Assembly.ManifestModule.Name.Replace(".dll", "");
            this.FullName = Assembly.FullName;

            foreach (var type in assembly.GetTypes())
            {
                if (type != CONNECTION_TYPE && CONNECTION_TYPE.IsAssignableFrom(type))
                    this.ConnectionType = type;
            }
        }

        /// <summary>
        /// Does this dll implement a IDbConnection interface
        /// </summary>
        /// <param name="assembly">The dll to check</param>
        /// <returns>Boolean success/failure</returns>
        public static bool IsDatabaseProvider(Assembly assembly)
        {
            bool returnValue = false;

            try
            {
                foreach (var type in assembly.GetTypes())
                {
                    returnValue = (type != CONNECTION_TYPE && CONNECTION_TYPE.IsAssignableFrom(type));

                    if (returnValue)
                        break;
                }
            }
            catch (Exception)
            {
                //We can ignore this
            }

            return returnValue;
        }

        /// <summary>
        /// Creates an instance of a connection
        /// </summary>
        /// <returns>The connection object</returns>
        public IDbConnection CreateConnection()
        {
            IDbConnection returnValue = (IDbConnection)Activator.CreateInstance(this.ConnectionType);

            if (returnValue == null)
                throw new Exception("Unable to initialize connection for " + this.Name);

            return returnValue;
        }
    }
}
